// Item.h
// Header File for the CItem class
////////////////////////////////////////

#ifndef CITEM
#define CITEM

#ifdef _DLL
//	#include "DLL_CONST.h"
#else
	#include <afx.h>
#endif

// Include version info
#include "..\Versions\versions.h"

#include "BBox.h"

#include "PosArray.h"

#include "HatchFill.h"

/*** BARCODE ***/
#include "BarCode.h"

//#define CI_VERSION	100

// File Name Extensions
#define	FONT_EXT	"ALF"
#define FONT_FLTR	"*.ALF"
#define OBJ_EXT		"ALF"
#define OBJ_FLTR	"*.ob2"
#define GRPH_EXT	"ALG"
#define GRPH_FLTR	"*.ALG"

// X/Y Justifications
#define JX_LEFT		0
#define JX_CENTER	1
#define JX_RIGHT	2
#define JY_BOTTOM	0
#define JY_CENTER	1
#define JY_TOP		2

// Item Types
#define CI_NUM_TYPES	8
#define CI_NO_TYPE	0
#define CI_FIXED	1
#define CI_VARTEXT	2
#define CI_GRAPHIC	3
#define CI_EMPTY	4
#define CI_LINK		5
#define CI_LINE		6
#define CI_SPOT		7

// Text
#define CI_NUMTEXTLINES		10

// Variable Text Types
#define CI_NOVARTEXT		32000
#define CI_KEYBOARD			0
#define	CI_DATE				1
#define	CI_CODE_READER  	2
#define CI_SERIAL_NUMBER    3
#define CI_FILE				4

// Mark Info
#define CI_PASSES_MIN		1
#define CI_PASSES_MAX		7

#define CI_PRCNT_MIN		0
#define CI_PRCNT_MAX		2000
#define CI_WOBBLE_MIN		0
#define CI_WOBBLE_MAX		1500

// Font Types
#define	CI_FONT_ENGRAVING	0
#define	CI_FONT_TRUETYPE	1
#define	CI_FONT_BARCODE		2

/*** DAN JAN 22 ***/
// Prompt timing
#define CI_PROMPT_PART		0
#define CI_PROMPT_BATCH		1
#define CI_PROMPT_NEVER		2
/******************/

/*** 1.0067 PAT 02.06.98 MORE RELATIVE / ABSOLUTE PATH ***/
#define ITEM_DEFAULT_FONT				"SIMPLEX.ALF"
#define ITEM_DEFAULT_GRAPHIC			"BOX.ALG"
#define ITEM_DEFAULT_BOX				"BOX.ALG"
#define	ITEM_DEFAULT_CIRCLE				"CIRCLE.ALG"
/*********************************************************/

// Default font names
/*** PAT 6 JAN 1998 ***/
//#define CI_ENGRAVING_DEFAULT	"C:\\DEV40\\WINSCRIBE\\SIMPLEX.ALF"
#define	CI_ENGRAVING_DEFAULT	"SIMPLEX.ALF"
/***/

#define CI_TRUETYPE_DEFAULT		"Arial#____+05+00.TAT"
/*** BARCODE ***/
#define	CI_BARCODE_DEFAULT		"Code 39.ABF"

/*** 1.0105 PAT 02.09.98 NEW TEXT ASPECT MODES ***/
#define CI_ASPECTRATIO_RES			100
#define CI_ASPECTMODE_FREE			0
#define CI_ASPECTMODE_NATURAL		1
#define CI_ASPECTMODE_FIXED			2
#define CI_ASPECTMODE_SQUEEZE		3
/*************************************************/

//#ifdef _DLL
//class __declspec( dllexport ) CItem : public CObject
//#else
class CItem : public CObject
//#endif
{


	DECLARE_SERIAL( CItem )

//Attributes
public:
	
	// CItem class version #
//	int		m_iVersion;
	UINT	m_nVersion;

	UINT	m_bItemUsed;		// Flag to tell if an item is being used
	
// Variables for linked item
	UINT	m_bLinkedItem;		// Flag to tell if item is linked
	CString	m_sItemFileName;	// Linked Item file name

// Item Identification
	CString m_sItemName;		// Name of the item (user defined)


// Item Position / Orientation
	long	m_lXPos,			// X Position
			m_lYPos,			// Y Position
			m_lHeight,			// Height
			m_lLength;			// Length

	int		m_iAngle,			// Rotation Angle
			m_iItalic;			// Italic Angle

	long	m_lRadius;			// Ring Mode Radius

	int		m_nXJust,			// X Justification
			m_nYJust;			// Y Justification

	BOOL	m_bMirrorX,			// X Mirror
			m_bMirrorY;			// Y Mirror

	/*** 1.0105 PAT 02.09.98 NEW TEXT ASPECT MODES ***/
	// This variable is obsolete
	/*** PAT 6 DEC 97 ***/
	//BOOL	m_bFixedAspect;
	/***/
	// Variables for the new text modes
	UINT
		m_nAspectMode;
	/*** 1.0113 PAT 21.09.98 ASPECT MODE ADJUSTMENTS ***/
	// Made this variable a double
	//int
	//	m_iAspectRatio;
	float
		m_fAspectRatio;
	/***************************************************/

	/*************************************************/	


	// Item Type Information
	UINT	m_nItemType;		// Item Type

	// Text Specific Information
	CString	m_sText[CI_NUMTEXTLINES];		// Text Lines
	CString	m_sFixedText[CI_NUMTEXTLINES];	// Fixed Text Data
	CString	m_sFont;			// FontName

	long	m_lCharSpace;		// Character Spacing
	long	m_lLineSpace;		// Line Spacing


	// Variable Text Information
	int		m_iVarTextType;		// Variable Text Type

	CString	m_sVarPrompt;		// Prompt for keyboard input or code reader
	CString m_sVarSample;
	CString	m_sVarFileName;		// FileName for variable text input
	CString m_sVarSerialNum;
	CString m_sVarStartSerialNum;
	int		m_iVarSerialInc;	// Serial Increment
	long	m_lVarSerialNum;	// Serial Number for variable text input
	int		m_iVarDateCodeType;	// Date code style for variable text input
	/*** DAN BARCODE ***/
	CString m_sVarDateCode;		// Date code string
	/*******************/

	// Added by Keith 1/10/98
	BOOL	m_bVarSerializeObjectArrays;
	BOOL	m_bVarResetPart;
	BOOL	m_bVarResetLimit;
	BOOL	m_bVarResetDateChange;
	BOOL	m_bVarResetBatch;

	CString	m_sVarResetLimitValue;
	int		m_iVarMarksPerIncrementValue;

	/*** 1.0123 PAT 13.10.98 SUPRESS LEADING ZEROS ***/
	BOOL
		m_bVarSerialSupressZeros;
	/*** 1.0123 **************************************/

	/*** 1.0125 PAT 13.10.98 FIX DATALINK TO BARCODE ***/
	CString
		m_sVarText;
	/*** 1.0125 ****************************************/

	/* DAN JAN 22 **/
	int		m_iPrompt;
	/***************/

	// Item array information
	UINT	m_nXArray,			// X Dimension
			m_nYArray;			// Y Dimension
	long	m_lXInc,			// X Increment
			m_lYInc;			// Y Increment
	UINT	m_bLocalCoord;		// Local Coordinates Flag
	int		m_iCharSet;

	// Graphic Specific Information
	CString	m_sFileName;		// Graphic File Name

	// Linked Item Specific Information
	CString	m_sLinkItemName;
	int	m_iLinkLayer,
		m_iLinkItem;


	// Item Variable Mark Informatiom

	unsigned char	m_uCondMark,	// Item Conditional Mark
					m_uCondMask;	// Item Conditional Mask

	int		m_iSpeedFac,		// Speed Percentage Factor
			m_iPowerFac,		// Power Percentage Factor
			m_iFreqFac;			// Frequency Percentage Factor

	long	m_lWobbleWidth;		// Wobble Width
	int		m_iWobbleType;
	int		m_iWobbleDensity;

	int		m_iPasses;			// Number of Passes

	float m_fSpeed2;
	float m_fPower2;
	float m_fFreq2;
	BOOL m_bUseMaterialFile;


// Bounding Box information
	CBBox	m_BBox;

	BOOL	m_bCanBeFilled;		// can it have a fill
	BOOL	m_bUseOutline;		// Include the outline
	BOOL	m_bUseFill;			// Include the fill
	CHatchFill m_HatchFill;	// Hatch Fill vectors

/*** BARCODE ***/
/*** PAT 25 FEB 1998 ***/
	BOOL	m_bUseBarCode;
	CBarCode m_BCData;
/***/

// Item Specific Delays
	UINT	m_bProgramValues;	// Flag to use program values
	UINT	m_nResPos;			// Delays and timer values
	UINT	m_nTcAngle;
	UINT	m_nTcDistance;
	UINT	m_nTcEndSeg;
	UINT	m_nTcPoly;
	UINT	m_nTcPos;
	UINT	m_nTcStartSeg;

//	Temporary for add on without version change
	/*** 1.0001 DAN 4/16/98 LIST FILE SUPPORT ***/
	// Used one of the temp integers
	///int  m_iTemp1;
	int	m_iVarFileRecord;
	/********************************************/

	int  m_iTemp2;
	int	 m_iTemp3;
	int	 m_iTemp4;
	int	 m_iTemp5;

	CString  m_sTemp1;
	CString	 m_sTemp2;
	CString	 m_sTemp3;
	CString	 m_sTemp4;
	CString	 m_sTemp5;

	/*** 1.0117 PAT 08.10.98 MODIFIED LENGTH FLAG ***/
	// Replaced one temp flag
	//BOOL	 m_bTemp1;
	BOOL
		m_bModifiedLength;
	/*** 1.0117 **************************************/
	BOOL	 m_bTemp2;
	BOOL	 m_bTemp3;
	BOOL	 m_bTemp4;
	BOOL	 m_bTemp5;


private:

	int	m_iMarkCount;

	


// Operators
public:
	CItem(void);		// Constructor
	~CItem(void);		// Destructor

 /*** PAT 1 FEB 1998 ***/
	// equate operator =
	void operator =( const CItem& src );
	// equivalence operator ==
	BOOL operator ==( const CItem& src );
/***/


	void Init(void);	// Initialization


	
    void Serialize( CArchive& ar );
	// Resets Serial number to the Starting Number
	void ResetSerialText (void);
	// Resets MarkCount
	void ResetMarkCount (void);
	// Increments the Serial Text of an item
	void IncrementSerialText (void);
	/*** 1.0096 10.22.98 SERIAL NUMBER OVERFLOW ***/
	// Increments a single character, return TRUE if it flipped
	UINT IncrementChar (unsigned char *ch, int iInc);
	/****1.0096************************************/


	// Delete the fill
	void DeleteFill(void);

	// Sets the output text
	void SetOutputText(void);

	/*** 1.0025 PAT 09.05.98 VERIFY FILES ***/
	// Ensures that the item's file references are valid
	void ConfirmLinks(int, int);
	// Obtains a new file name for an item
	BOOL ChooseNewFile(CString&, int);
	/***************************************/

	/*** 1.0053 PAT 21.05.98 RELATIVE/ABSOLUTE PATHS ***/
	CString MakeRelative(CString);
	CString MakeAbsolute(CString);
	/***************************************************/

	/*** 1.0123 PAT 13.10.98 SUPRESS LEADING ZEROS ***/
	CString RemoveLeadingZeros(CString);
	/*** 1.0123 **************************************/
};


#endif












