// LM45Share.h

// Header file for the CLM45Share class.
//
// Pat Summers
// American LaserWare, Inc.

#ifndef	LM45SHARE
#define LM45SHARE

// Include all revision notes
#include "LM45Share_Ver.h"
// Include definitions of constants used by this class
#include "LM45Share_Constants.h"

// Define the class
class CLM45Share : public CObject
{
public:
	// Version number for class
	UINT
		m_nVersion;
	// Constructor
	CLM45Share(void);
	// Destructor
	~CLM45Share(void);
	// Initalization for class
	BOOL Init(BOOL bManager = FALSE);
	// Clean up and disposal
	void Outit(void);

private:
	// Shared data
	LM45ShareData
		m_ShareData;
public:
	// References to shared memory
	HANDLE
		m_hShareFileMapping;
	LPVOID
		m_lpShareAddress;
	HANDLE
		m_hShareMutex;
	// Shared memory control
	int
		m_iShareID;
	BOOL
		m_bSharing,
		m_bIsManager,
		m_bTheBoss,
		m_bTheADCBoss;
	// Shared memory access
	void ShareIn(void);
	void ShareOut(void);
	void InitSharing(void);
	BOOL CanModifyHardware(void);
	int	AddRef(void);
	void DelRef(void);
	int AddManager(void);
	void DelManager(void);
	BOOL IsManaged();
	BOOL Promote(int iTimeOut = 1);
	void Demote(void);

	// Maintenance
	int GetCount(void);
	void SetCount(int);
	int GetManCount(void);
	void SetManCount(int);
	BOOL IsBoardInitDone(void);
	void SetBoardInitDone(BOOL);
	// Access Control
	int GetBoss(void);
	void SetBoss(int);
	int GetADCBoss(void);
	void SetADCBoss(int);
	BOOL IsShuttingDown(void);
	void SetShuttingDown(BOOL);
	BOOL IsStartingUp(void);
	void SetStartingUp(BOOL);
	// Ports
	WORD GetPort0202(void);
	void SetPort0202(WORD);
	WORD GetPort0204(void);
	void SetPort0204(WORD);
	WORD GetPort0218(void);
	void SetPort0218(WORD);
	WORD GetPort021C(void);
	void SetPort021C(WORD);
	WORD GetPort0224(void);
	void SetPort0224(WORD);
	// Gage values
	float GetWaterTemp(void);
	void SetWaterTemp(float);
	float GetWaterCond(void);
	void SetWaterCond(float);
	float GetPSLampCurrent(void);
	void SetPSLampCurrent(float);
	// Settings
	float GetQSDFreqSetting(void);
	void SetQSDFreqSetting(float);
	float GetQSDModFactorSetting(void);
	void SetQSDModFactorSetting(float);
	float GetPSLampSetting(void);
	void SetPSLampSetting(float);

	// LM45
	int GetState(void);
	int SetState(int);
	int GetCurrState(void);
	int SetCurrState(int);
	BOOL GetLM45Ready(void);
	BOOL SetLM45Ready(BOOL);
	BOOL IsLM45Ready(void);
	/*** REV S001 - 19990606 - PROLASE-LIKE IO ***/
	BOOL GetLM45Good(void);
	void SetLM45Good(BOOL);
	BOOL IsLM45Good(void);
	/*** REV S001 ************************************/
	BOOL GetAnyReset(void);
	void SetAnyReset(BOOL);
	BOOL IsAnyReset(void);
	BOOL GetAnyStart(void);
	void SetAnyStart(BOOL);
	BOOL IsAnyStart(void);
	BOOL GetAnyStop(void);
	void SetAnyStop(BOOL);
	BOOL IsAnyStop(void);
	// Mark prog
	BOOL GetMarkProgSet(void);
	void SetMarkProgSet(BOOL);
	BOOL IsMarkProgSet(void);
	BOOL GetMarkProgInMarkMode(void);
	void SetMarkProgInMarkMode(BOOL);
	BOOL IsMarkProgInMarkMode(void);
	BOOL GetMarkProgReady(void);
	void SetMarkProgReady(BOOL);
	BOOL IsMarkProgReady(void);
	BOOL GetMarkProgMarking(void);
	void SetMarkProgMarking(BOOL);
	BOOL IsMarkProgMarking(void);
	BOOL GetMarkProgLaserOn(void);
	void SetMarkProgLaserOn(BOOL);
	BOOL IsMarkProgLaserOn(void);
	BOOL GetMarkProgDone(void);
	void SetMarkProgDone(BOOL);
	BOOL IsMarkProgDone(void);
	BOOL GetMarkProgCounterUp(void);
	void SetMarkProgCounterUp(BOOL);
	BOOL IsMarkProgCounterUp(void);
	BOOL GetMarkProgAborted(void);
	void SetMarkProgAborted(BOOL);
	HWND GetMarkProgWindow(void);
	void SetMarkProgWindow(HWND);
	BOOL IsMarkProgWindow(void);
	BOOL IsMarkProgAborted(void);

	
	
	int GetSerialCounter(void);
	void SetSerialCounter(int);
	int IncSerialCounter(void);
	unsigned short GetScheduleNumber(void);
	void SetScheduleNumber(unsigned short);

	/*** REV S001 - 19990606 - PROLASE-LIKE IO ***/
	BOOL GetMarkProgReadyLED(void);
	void SetMarkProgReadyLED(BOOL);
	BOOL IsMarkProgReadyLED(void);
	/*** REV S001 ************************************/

	/*** REV S002 - 19990610 - LAMP POWER LOCK ***/
	BOOL GetLampPowerLocked(void);
	void SetLampPowerLocked(BOOL);
	BOOL IsLampPowerLocked(void);
	/*** REV S002 ********************************/

	/*** REV S004 - 19990618 - MARK PROG MARKING NOW FLAG ***/
	BOOL GetMarkProgMarkingActive(void);
	void SetMarkProgMarkingActive(BOOL);
	BOOL IsMarkProgMarkingActive(void);
	/*** REV S004 *******************************************/

	/*** REV S005 - 19990715 - BOARD FAULTY VARIABLE ***/
	BOOL GetBoardFaulty(void);
	void SetBoardFaulty(BOOL);
	BOOL IsBoardFaulty(void);
	/*** REV S005 **************************************/

};

#endif	// LM45


