// Header file for the CRTC3Data class
//
// RTC3Data.h

// This class is the data structure for holding data concerning the
// configuration of the RTC3 board


//// 1.0445 PAT 11.10.00 RTC3 DRIVER ////
// Added this file
//// 1.0445 /////////////////////////////


#ifndef CRTC3DATA
#define CRTC3DATA

#include <afx.h>

#define RTC3DATA_2000_10_11			0
//// 1.0503 PAT 22.02.01 RTC3 LASER EXTENSION ////
#define RTC3DATA_2001_02_22			10
//// 1.0503 //////////////////////////////////////
//// 1.0533 PAT 02.05.01 RTC I/O PATCH ////
#define RTC3DATA_2001_05_02			20
//// 1.0533 ///////////////////////////////
//// 1.0651 PAT 02.04.02 RTC3 LASER EXT GP DIG OUT ////
#define RTC3DATA_2002_04_02			30
//// 1.0651 ///////////////////////////////////////////
//// 1.0897 PAT 17.06.04 RTC DELAYS ADDIN ////
// Adds a number of values relating to mark/move delay options
#define RTC3DATA_2004_06_17			40
//// 1.0897 //////////////////////////////////
//// 1.1027 PAT 08.06.05 RTC LOGGING ////
#define RTC3DATA_2005_06_08			50
//// 1.1027 /////////////////////////////

//// 1.1263 PAT 13.11.06 RTC FOCUS CONTROL ////
#define RTC3DATA_2006_11_13			60
//// 1.1263 ///////////////////////////////////

//// PS090417A RTC CALL LOGGING FILTERING ////
#define RTC3DATA_2009_04_17			70
//// PS090417A ///////////////////////////////

#define VER_RTC3DATA					RTC3DATA_2009_04_17
//	#define VER_RTC3DATA				RTC3DATA_2006_11_13
//	#define VER_RTC3DATA				RTC3DATA_2005_06_08
//	#define VER_RTC3DATA				RTC3DATA_2004_06_17
//	#define VER_RTC3DATA				RTC3DATA_2002_04_02
//	#define VER_RTC3DATA				RTC3DATA_2001_05_02
//	#define VER_RTC3DATA				RTC3DATA_2001_02_22
//	#define VER_RTC3DATA				RTC3DATA_2000_10_11


#ifdef _ALIUSE
	#include ".\RTC3Constants.h"
	#include ".\RTC3CTBXform.h"
#else
	#include "RTC3Constants.h"
	#include "RTC3CTBXform.h"
#endif

class CRTC3Data : public CObject
{

	DECLARE_SERIAL( CRTC3Data )
private:
// Attributes
	// Version info
	UINT
		m_nVersion;
public:
	CString
		m_sDLLFileName,
		m_sProgFileName;
	UINT
		m_nCTBs;
	BOOL
		m_bCTBEnable[RTC3_CTB_NUM];
	CString
		m_sCTBFileName[RTC3_CTB_NUM];
	CRTC3CTBXform
		m_CTBXform[RTC3_CTB_NUM];

	UINT
		m_nHeads;
	UINT
		m_nHeadCTB[RTC3_HEAD_NUM];
	
	UINT
		m_nLaserType;

	UINT
		m_nArcRes;

	BOOL
		m_bVariableAngleDelay,
		m_bVariableDirDelay,
		m_bMaxPolyDelay;
	double
		m_dMaxPolyDelay;

	// On-The-Fly
	BOOL
		m_bFlyEnableX,
		m_bFlyEnableY;
	double
		m_dFlyCPI_X,
		m_dFlyCPI_Y;
	
	// 3D
	BOOL
		m_b3DZCal;
	double
		m_d3DZCalA,
		m_d3DZCalB,
		m_d3DZCalC;

	//// 1.0503 PAT 22.02.01 RTC3 LASER EXTENSION ////
	BOOL
		m_bLaserExtInvert;
	//// 1.0503 //////////////////////////////////////

	//// 1.0533 PAT 02.05.01 RTC I/O PATCH ////
	BOOL
		m_bEnableExtStartStop,
		m_bEnableIOExtension;
	//// 1.0533 ///////////////////////////////

	//// 1.0651 PAT 02.04.02 RTC3 LASER EXT GP DIG OUT ////
	int
		m_iDIOMode;
	//// 1.0651 ///////////////////////////////////////////

	//// 1.0897 PAT 17.06.04 RTC DELAYS ADDIN ////
	BOOL
		m_bDirectMove3D;
	UINT
		m_nJumpMin;
	BOOL
		m_bJumpLimitEnabled;
	UINT
		m_nJumpLimit;

	BOOL
		m_bCustomPolyEnabled;
	CString
		m_sCustomPolyFilename;
	unsigned short
		m_nCustomPolyTable;
	//// 1.0897 //////////////////////////////////

	//// 1.1027 PAT 08.06.05 RTC LOGGING ////
	BOOL m_bLogEnabled;
	int	m_iLogMode;
	//// 1.1027 /////////////////////////////

	//// 1.1263 PAT 13.11.06 RTC FOCUS CONTROL ////
	int m_iAnalog1Mode;
	//// 1.1263 ///////////////////////////////////

	//// PS090417A RTC CALL LOGGING FILTERING ////
	BOOL m_bLogAllStatus;
	//// PS090417A ///////////////////////////////

// Implementations
public:
	CRTC3Data(void);
	~CRTC3Data(void);

	void Init(void);

    void Serialize( CArchive& ar );

// Operators
	void operator =( const CRTC3Data& src );
	BOOL operator ==( const CRTC3Data& src );
public:
};

#endif // CRTC3DATA

