// RTC3Driver.h 
//
// Header file for the CRTC3Driver class
//////////////////////////////////////////

//// 1.0445 PAT 11.10.00 RTC3 DRIVER ////
// Added this file
//// 1.0445 /////////////////////////////
 
#ifndef RTC3DRIVER
#define RTC3DRIVER

#ifdef _ALIUSE
	#include ".\WSData\RTC3Data.h"
	//// JT080128A Adding Custom Pulse Width Table for RTC4 ////
	#include ".\WSData\Config.h"
	//// JT080128A /////////////////////////////////////////////
#else
	#include "RTC3Data.h"
	//// JT080128A Adding Custom Pulse Width Table for RTC4 ////
	#include "Config.h"
	//// JT080128A /////////////////////////////////////////////
#endif

#include ".\RTC3State.h"
#include ".\RTC3DLL_TYPE.h"


// Class specific constants
const double	RTC3_CLOCK_PER_SEC	=	1.193180;


//// 1.1027 PAT 08.06.05 RTC LOGGING ////
#define		RTC3_LOGFILE_MAXFILES	100
#define		RTC3_LOGFILE_MAXCALLS	10000
#define		RTC3_LOGFILE_FOLDER		_T("RTC3Logs")
#define		RTC3_LOGFILE_NAME		_T("RTC3Log")
//// 1.1027 /////////////////////////////

class CRTC3Driver
{
public:
// Attributes

//// 1.1027 PAT 08.06.05 RTC LOGGING ////
#include "RTC3DriverWrap_DEC.h"
//// 1.1027 /////////////////////////////

	//// 1.0531 PAT 26.04.01 RTC ABORT ADDITIONS ////
	CObject
		*m_pMark;
	//// 1.0531 /////////////////////////////////////

protected:
	CRTC3Data
		m_Data;
	CRTC3State
		m_State;

	BOOL
		m_bRTC3Working;
	
	HINSTANCE
		m_gLibRTC3;
	
	BOOL
		m_bDLLPresent,
		m_bBoardPresent,
		m_bCTBPresent[RTC3_CTB_NUM],
		m_bHexPresent;

	CString
		m_sLoadedDLLFileName;

	double
		m_dFieldSize;
	BOOL
		m_bFieldSwapXY,
		m_bFieldNegateX,
		m_bFieldNegateY;

	double
		m_dQSWidth,			// microseconds
		m_dFPSWidth;		// microseconds
	double
		m_dTicklePeriod,	// microseconds
		m_dTickleWidth;		// microseconds
	
	double
		m_dPowerFraction;
	double
		m_dPeriod,			// microseconds
		m_dPulseWidth;		// microseconds

	double
		m_dJumpTC,			// microseconds
		m_dMarkTC,			// microseconds
		m_dPolyTC,			// microseconds
		m_dLaserOnTC,		// microseconds
		m_dLaserOffTC,		// microseconds
		m_dMaxPolyTC;		// microseconds

	double
		m_dJumpSpeed,		// units per second
		m_dMarkSpeed;		// unsit per second

	// List Control Flags
	UINT
		m_nListSize,
		m_nListFlushLimit,
		m_nMaxListSize;
	BOOL
		m_bUsingList1;
	UINT
		m_nArcPolyLength;
	UINT
		m_nPrevPeriod,
		m_nDelay;

	UINT
		m_nListPosition;

	short
		m_siLastX,
		m_siLastY,
		m_siLastZ;

	unsigned short
		m_nRasterPeriod;
	UINT
		m_nRasterListSize;

	//// 1.0501 PAT 21.02.01 RTC DIO ////
	unsigned short
		m_usDigOut;	// Shadow for output port
	//// 1.0501 /////////////////////////

	//// 1.0523 PAT 06.04.01 MORE RTC3 RASTER ////
	BOOL
		m_bInRasterMode;
	//// 1.0523 //////////////////////////////////

	//// 1.0531 PAT 26.04.01 RTC ABORT ADDITIONS ////
	BOOL
		m_bAborted;
	//// 1.0531 /////////////////////////////////////

	//// 1.0561 PAT 20.06.01 RTC WOBBLE ADDITIONS ////
	double
		m_dWobbleAmp,
		m_dWobbleFreq;
	//// 1.0561 //////////////////////////////////////


// Operators
public:

	CRTC3Driver (void);
	~CRTC3Driver (void);

	void Init (void);

	UINT InitCard(void);

	void GetData(CRTC3Data *pData);
	BOOL SetData(CRTC3Data *pData);
	void GetState(CRTC3State *pState);
	void UpdateState(void);
	BOOL IsWorking(void);
	UINT GetCardCount(void);
	UINT GetCurrentCard(void);
	UINT SetCurrentCard(UINT nCard);
	UINT GetVersionHardware(void);
	UINT GetVersionSoftware(void);
	UINT GetVersionDLL(void);
	UINT GetVersionSerialNumber(void);
	void GetOptions(void);
	BOOL IsOptFlyCapable(void);
	BOOL IsOpt2HeadsCapable(void);
	BOOL IsOpt3DCapable(void);
	BOOL IsOptIOCapable(void);

	int LoadCorrectionFile(UINT nTable, BOOL bNoGain = TRUE);
	int LoadAllCorrectionFiles(void);
	void SetCorrectionTables(BOOL bSetup = FALSE);
	int LoadProgramFile(void);

	void SetControlMode(void);
	void SetListMode(void);
	UINT IsBusy(void);
	//// 1.0531 PAT 26.04.01 RTC ABORT ADDITIONS ////
	//	void WaitOnList(UINT nList = 0);
	BOOL WaitOnList(UINT nList = 0);
	//// 1.0531 /////////////////////////////////////
	void Abort(void);
	UINT Pause(int iDelay = -1);
	void SetDelayMode(void);
	void SetScannerDelays(void);
	void SetScannerSpeed(void);
	void SetLaserMode(void);
	void SetLaserDelays(void);
	//// 1.0523 PAT 06.04.01 MORE RTC3 RASTER ////
	//	void SetLaserStandby(void);
	//	void SetLaserFPS(void);
	void SetLaserStandby(BOOL bOn);
	void SetLaserFPS(BOOL bOn);
	//// 1.0523 //////////////////////////////////
	void SetLaserTiming(void);
	void SetAnalogPort(unsigned short nPort, unsigned short nValue);
	void SetDIOPort(BYTE byValue);

	void CenterGalvos(void);
	//// 1.0587 PAT 15.08.01 RTC3 FLY ISSUES ////
	//	void LaserOn(double dOnTime = 1000.0);
	void LaserOn(double dOnTime = 0.0);
	//// 1.0587 /////////////////////////////////
	void LaserOff(void);

	//// 1.0531 PAT 26.04.01 RTC ABORT ADDITIONS ////
	//	void OpenList (void);
	BOOL OpenList (void);
	//// 1.0531 /////////////////////////////////////
	void FlushList (void);

	//// 1.0531 PAT 26.04.01 RTC ABORT ADDITIONS ////
	//	void MoveGalvos (short *, short *, short *, short *, long *, short *);
	BOOL MoveGalvos (short *, short *, short *, short *, long *, short *);
	//// 1.0531 /////////////////////////////////////
	void AddMove (short siX, short siY);
	void AddLine (short siX, short siY);
	void AddArc(long *lVecInfo, short *siVecInfo);
	BOOL GetNextArcSegment(long *lVecInfo, short *siVecInfo, short *siX, short *siY, UINT m_nArcPolyLength);
	void TransformXYZ(short *psiX, short *psiY, short *psiZ);
	//// 1.0625 PAT 07.01.02 DRILL RASTER ////
	void TransformXYZ(double *pdX, double *pdY, double *pdZ);
	//// 1.0625 //////////////////////////////
	//// 1.0531 PAT 26.04.01 RTC ABORT ADDITIONS ////
	//	void MoveGalvosRaster(short *, short *, short *, short *, long *, short *, BYTE *);
	//	void MoveGalvosRaster8(short *, short *, short *, short *, long *, short *, BYTE *, BYTE *);
	BOOL MoveGalvosRaster(short *, short *, short *, short *, long *, short *, BYTE *);
	BOOL MoveGalvosRaster8(short *, short *, short *, short *, long *, short *, BYTE *, BYTE *);
	//// 1.0531 /////////////////////////////////////
	void AddRasMove (short siX, short siY);
	void AddRasDot (short siDX, short siDY, short siSteps, unsigned short *pwData);
	void Spot(short *, short *, short *);

	void GotoPos(short x, short y, short z = 0);
	void GetPos(short *x, short *y, short *z = NULL);
	void JumpAbs(short x, short y, short z = 0);
	void MarkAbs(short x, short y, short z = 0);
	void SetPixelLine(short dx, short dy, unsigned short N, unsigned short Period, unsigned short *Ptr);
	void SetPixel(unsigned short Data, unsigned short ADC);

	void GetLastPos(short *x, short *y, short *z = NULL);
	void SetLastPos(short x, short y, short z = 0);
	
	unsigned short SetRasterSpeed(UINT nPeriod);
	UINT SetRasterListSize(UINT nPixels);

	void SetConfigValues(CRTC3Data *pData);
	void SetConfigFieldSize(double dSize);
	void SetConfigDelayValues(double dJumpTC, double dMarkTC, double dPolyTC, double dLaserOffTC, double dLaserOnTC);
	void SetConfigLaserValues(double dTicklePeriod, double dTickleWidth, double dQSWidth, double dFPSWidth);
	void SetSpeedValues(double dJump, double dMark);
	//// 1.0539 PAT 14.05.01 RTC2 MIN SPEED FIX ////
	void SetSpeed(int iPosRes, double dFieldSize, double dMarkSpeed);
	//// 1.0539 ////////////////////////////////////
	void SetPowerAndPeriod(double dFraction, double dPeriod);
	void SetLaserPower(double dFraction);
	void SetDelays(double dStart, double dPoly, double dAngle, double dEnd, double dPos, double dDist,double dDistance);
	
	BOOL GetFlyCounts(long *pCountsX, long *pCountsY);
	BOOL SetFlyKX(double *pX);
	BOOL SetFlyKY(double *pY);
	//// 1.0587 PAT 15.08.01 RTC3 FLY ISSUES ////
	//	void SetEncoderValues(void);
	void SetEncoderValues(BOOL bEnable);
	//// 1.0587 /////////////////////////////////

	double Get3DZOffset(void);
	void Set3DZOffset(double dZOffset);

	//// 1.0499 PAT 19.02.00 RTC START/STOP ////
	BOOL IsExtStartSignal(void);
	BOOL IsExtStopSignal(void);

	BOOL EnableExtStartSignal(BOOL bEnable);
	BOOL SetNullLists(void);
	BOOL ClearExtStopSignal(void);
	//// 1.0499 ////////////////////////////////

	//// 1.0501 PAT 21.02.01 RTC DIO ////
	BOOL ReadDigIns(BYTE *pIn);
	BOOL WriteDigOuts(BYTE byOut);
	BOOL GetDigOuts(BYTE *pOut);
	//// 1.0501 /////////////////////////

	//// 1.0503 PAT 22.02.01 RTC3 LASER EXTENSION ////
	BOOL SetLaserExtension(BYTE byValue);
	//// 1.0503 //////////////////////////////////////

	//// 1.0523 PAT 06.04.01 MORE RTC3 RASTER ////
	void SetLaserTickle(BOOL bOn);
	void SetRasterMode(BOOL bOn);
	//// 1.0523 //////////////////////////////////

	//// 1.0533 PAT 02.05.01 RTC I/O PATCH ////
	BOOL IsExtStartStopEnabled(void);
	BOOL IsIOExtensionEnabled(void);
	//// 1.0533 ///////////////////////////////

	//// 1.0537 PAT 09.05.01 RTC FLUSH LIST BUG ////
	UINT GetListPosition(void);
	BOOL IsInList1(void);
	BOOL IsInList2(void);
	UINT GetList(void);
	//// 1.0537 ////////////////////////////////////


	//// 1.0561 PAT 20.06.01 RTC WOBBLE ADDITIONS ////
	void SetWobble(BOOL bOn);
	void SetWobbleValues(double dWidth, double dFreq);
	//// 1.0561 //////////////////////////////////////

	//// 1.0581 PAT 06.08.01 RTC3 FIXES ////
	BOOL IsCTBFile3D(CString sPath);
	//// 1.0581 ////////////////////////////

	//// 1.0587 PAT 15.08.01 RTC3 FLY ISSUES ////
	BOOL IsFlyXEnabled(void);
	BOOL IsFlyYEnabled(void);

	int GetFlySimAxis(void);
	void SetFlySimAxis(int iAxis);

	void ResetFlyEncoder(BOOL bReset);
	//// 1.0587 /////////////////////////////////

	//// 1.0625 PAT 07.01.02 DRILL RASTER ////
	UINT MarkSpecialRaster(CObject *pItem, long lPosX, long lPosY);
	void JumpRel(short x, short y, short z = 0);
	void MarkRel(short x, short y, short z = 0);
	//// 1.0625 //////////////////////////////

	//// 1.0631 PAT 07.02.02 RTC3 FOCUS CONTROL ////
	BOOL SetFocus(unsigned short usFocus);
	//// 1.0631 ////////////////////////////////////

	//// 1.0651 PAT 02.04.02 RTC3 LASER EXT GP DIG OUT ////
	int GetDIOMode(void);
	//// 1.0651 ///////////////////////////////////////////

	//// 1.0757 PAT 14.04.03 RTC3 OTF FIX ////
	int ClearEncoderValues(void);
	//// 1.0757 //////////////////////////////

	//// 1.0867 PAT 06.05.04 COM/DLL LASE/POS CONTROL ////
	void SetGalvoPosition(short sX, short sY);
	//// 1.0867 //////////////////////////////////////////

	//// 1.0897 PAT 17.06.04 RTC DELAYS ADDIN ////
	void LoadVarPolyDelay(void);
	//// 1.0897 //////////////////////////////////

	//// 1.0919 PAT 08.08.04 PORT C DIO FIX ////
	BOOL ReadDigInsEx(unsigned short *pIn, BOOL bOverride);
	BOOL WriteDigOutsEx(unsigned short byOut, BOOL bOverride);
	BOOL GetDigOutsEx(unsigned short *pOut, BOOL bOverride);
	//// 1.0919 ////////////////////////////////

	//// 1.1027 PAT 08.06.05 RTC LOGGING ////
	BOOL m_bLogging;
	int m_iLogFileIndex;
	int m_iLogFileCallCount;
	CStdioFile m_LogFiles[RTC3_LOGFILE_MAXFILES];
	void LogFileStart(void);
	void LogFileCreateFolder(void);
	void LogFilePurge(void);
	void LogFileAdvance(void);
	void LogFileOpen(void);
	void LogFileAdd(CString sFunc, CString Arg, CString Result);
	BOOL LogFileTest(CString sFunc);
	void LogFileClose(void);
	//// 1.1027 /////////////////////////////

	//// 1.1335 PAT 15.06.07 UPDATE SPOT CODE ////
	void SpotX(double dDuration);
	//// 1.1335 //////////////////////////////////

	//// JT080128A Adding Custom Pulse Width Table for RTC4 ////
	bool	m_bPTLeadingEdge;
	BOOL	m_bPulseTableOn;
	double	m_adPulseTable[PT_MAX_ENTRIES];
	int		m_iPulseTableArraySize;
	double	m_dRestartDelay;

	void SetConfigLaserPulseTable( BOOL bOn, int iLength, double *pArray, double dRestart, bool bLeadingEdge );
	void SetCustomPulseTable();
	//// JT080128A /////////////////////////////////////////////

	//// PS090924B RTC WAVEFORM SELECT ////
	int m_iLaserType;
	int SetLaserType(int iType);

	int m_iWaveform;
	BOOL	SetLaserWaveform(int iWaveform);
	//// PS090924B ////////////////////////

	//// PS080318A RTC Q WIDTH CONTROL ////
	double m_dQSWidthConfig;
	void SetLaserPeriodAndWidth(double dPeriod, double dWidth);
	//// PS080318A ////////////////////////

	//// PS080718A PULSE TABLE ANALOG MODE ////
	int m_iPTMode;
	double m_dPTFinal;
	void SetConfigLaserPulseTable2( BOOL bOn, int iMode, int iLength, double *pArray,
			double dRestart, bool bLeadingEdge , double dFinal);
	//// PS080718A ////////////////////////////

	//// PS090128A PREMOVE DELAY FOR DAC ////
	double	m_dPreJumpTC;	// microseconds
	void AddDelay(double dDelay);
	//// PS090128A //////////////////////////

	//// PS090417A RTC CALL LOGGING FILTERING ////
	int	m_iLogFilePrevCall;
	void LogFileEnd(void);
	//// PS090417A ///////////////////////////////

	//// PS090420A PREMOVE FOR RTC ////
	bool m_bLaserWasOn;
	//// PS090420A ////////////////////

	//// PS090924A ANALOG FREQ CONTROL IN RTC ////
	void SetLaserFreq(double dFraction);
	//// PS090924 ////////////////////////////////

	//// PS091203E ANALOG FREQ VS PULSE TABLE ////
	int GetPulseTablePort(void);
	//// PS091203E ///////////////////////////////

	//// PS20100427B SIMMER CURRENT SETTING ////
	void SetLaserPowerSimmerFraction(double dFraction);
	//// PS20100427B ///////////////////////////
protected:

#include ".\RTC3DLL_DEC.h"

protected:
};

#endif // RTC3DRIVER
