// Header file for the CRTC4Data class
//
// RTC4Data.h

// This class is the data structure for holding data concerning the
// configuration of the RTC4 board


//// 1.0835 PAT 02.03.04 RTC4 ////
// Added this file
//// 1.0835 //////////////////////


#ifndef CRTC4DATA
#define CRTC4DATA

#include <afx.h>

#define RTC4DATA_2004_03_02			0

//// 1.0847 PAT 08.04.04 RTC3/4 CABLE LENGTH SETTING ////
// Adds scanner cable lengths
#define	RTC4DATA_2004_04_08			1
//// 1.0847 /////////////////////////////////////////////

//// 1.0897 PAT 17.06.04 RTC DELAYS ADDIN ////
// Adds a number of values relating to mark/move delay options
#define RTC4DATA_2004_06_17			2
//// 1.0897 //////////////////////////////////

//// 1.1027 PAT 08.06.05 RTC LOGGING ////
#define RTC4DATA_2005_06_08			3
//// 1.1027 /////////////////////////////

//// 1.1263 PAT 13.11.06 RTC FOCUS CONTROL ////
#define RTC4DATA_2006_11_13			60
//// 1.1263 ///////////////////////////////////

//// PS090417A RTC CALL LOGGING FILTERING ////
#define RTC4DATA_2009_04_17			70
//// PS090417A ///////////////////////////////

#define VER_RTC4DATA					RTC4DATA_2009_04_17
//	#define VER_RTC4DATA				RTC4DATA_2006_11_13
//	#define VER_RTC4DATA				RTC4DATA_2004_06_17
//	#define VER_RTC4DATA				RTC4DATA_2005_06_08
//	#define VER_RTC4DATA				RTC4DATA_2004_03_02
//	#define	VER_RTC4DATA				RTC4DATA_2004_04_08


#ifdef _ALIUSE
	#include ".\RTC4Constants.h"
	#include ".\RTC4CTBXform.h"
#else
	#include "RTC4Constants.h"
	#include "RTC4CTBXform.h"
#endif

class CRTC4Data : public CObject
{

	DECLARE_SERIAL( CRTC4Data )
private:
// Attributes
	// Version info
	UINT
		m_nVersion;
public:
	CString
		m_sDLLFileName,
		m_sProgFileName;
	UINT
		m_nCTBs;
	BOOL
		m_bCTBEnable[RTC4_CTB_NUM];
	CString
		m_sCTBFileName[RTC4_CTB_NUM];
	CRTC4CTBXform
		m_CTBXform[RTC4_CTB_NUM];

	UINT
		m_nHeads;
	UINT
		m_nHeadCTB[RTC4_HEAD_NUM];
	
	UINT
		m_nLaserType;

	UINT
		m_nArcRes;

	BOOL
		m_bVariableAngleDelay,
		m_bVariableDirDelay,
		m_bMaxPolyDelay;
	double
		m_dMaxPolyDelay;

	// On-The-Fly
	BOOL
		m_bFlyEnableX,
		m_bFlyEnableY;
	double
		m_dFlyCPI_X,
		m_dFlyCPI_Y;
	
	// 3D
	BOOL
		m_b3DZCal;
	double
		m_d3DZCalA,
		m_d3DZCalB,
		m_d3DZCalC;

	BOOL
		m_bLaserExtInvert;

	BOOL
		m_bEnableExtStartStop,
		m_bEnableIOExtension;

	int
		m_iDIOMode;

	//// 1.0847 PAT 08.04.04 RTC3/4 CABLE LENGTH SETTING ////
	unsigned short
		m_nScannerCableA,
		m_nScannerCableB;
	//// 1.0847 /////////////////////////////////////////////

	//// 1.0897 PAT 17.06.04 RTC DELAYS ADDIN ////
	BOOL
		m_bDirectMove3D;
	UINT
		m_nJumpMin;
	BOOL
		m_bJumpLimitEnabled;
	UINT
		m_nJumpLimit;

	BOOL
		m_bCustomPolyEnabled;
	CString
		m_sCustomPolyFilename;
	unsigned short
		m_nCustomPolyTable;
	//// 1.0897 //////////////////////////////////

	//// 1.1027 PAT 08.06.05 RTC LOGGING ////
	BOOL m_bLogEnabled;
	int	m_iLogMode;
	//// 1.1027 /////////////////////////////

	//// 1.1263 PAT 13.11.06 RTC FOCUS CONTROL ////
	int m_iAnalog1Mode;
	//// 1.1263 ///////////////////////////////////

	//// PS090417A RTC CALL LOGGING FILTERING ////
	BOOL m_bLogAllStatus;
	//// PS090417A ///////////////////////////////

// Implementations
public:
	CRTC4Data(void);
	~CRTC4Data(void);

	void Init(void);

    void Serialize( CArchive& ar );

// Operators
	void operator =( const CRTC4Data& src );
	BOOL operator ==( const CRTC4Data& src );
public:
};

#endif // CRTC4DATA

