// RTCSA4Driver.h 
//
// Header file for the CRTCSA4Driver class
//////////////////////////////////////////

//// 1.1037 PAT 25.07.05 SCANALONE ////
// Added this file
//// 1.1037 ///////////////////////////
 
#ifndef RTCSA4DRIVER
#define RTCSA4DRIVER

#ifdef _ALIUSE
	#include ".\WSData\RTCSA4Data.h"
	//// JT080128A Adding Custom Pulse Width Table for RTC4 ////
	#include ".\WSData\Config.h"
	//// JT080128A /////////////////////////////////////////////
#else
	#include "RTCSA4Data.h"
	//// JT080128A Adding Custom Pulse Width Table for RTC4 ////
	#include "Config.h"
	//// JT080128A /////////////////////////////////////////////
#endif
#include ".\RTCSA4State.h"
#include ".\RTCSA4DLL_TYPE.h"


// Class specific constants
const double	RTCSA4_CLOCK_PER_SEC	=	1.193180;


#define		RTCSA4_LOGFILE_MAXFILES		100
#define		RTCSA4_LOGFILE_MAXCALLS		10000
#define		RTCSA4_LOGFILE_FOLDER		_T("RTCSA4Logs")
#define		RTCSA4_LOGFILE_NAME			_T("RTCSA4Log")

class CRTCSA4Driver
{
public:
// Attributes

#include "RTCSA4DriverWrap_DEC.h"

	CObject
		*m_pMark;

protected:
	CRTCSA4Data
		m_Data;
	CRTCSA4State
		m_State;


	BOOL
		m_bRTCSA4Working;
	
	HINSTANCE
		m_gLibSCANalone4;
	
	BOOL
		m_bDLLPresent,
		m_bBoardPresent,
		m_bCTBPresent[RTCSA4_CTB_NUM],
		m_bHexPresent;

	CString
		m_sLoadedDLLFileName;

	double
		m_dFieldSize;
	BOOL
		m_bFieldSwapXY,
		m_bFieldNegateX,
		m_bFieldNegateY;

	double
		m_dQSWidth,			// microseconds
		m_dFPSWidth;		// microseconds
	double
		m_dTicklePeriod,	// microseconds
		m_dTickleWidth;		// microseconds
	
	double
		m_dPowerFraction;
	double
		m_dPeriod,			// microseconds
		m_dPulseWidth;		// microseconds

	double
		m_dJumpTC,			// microseconds
		m_dMarkTC,			// microseconds
		m_dPolyTC,			// microseconds
		m_dLaserOnTC,		// microseconds
		m_dLaserOffTC,		// microseconds
		m_dMaxPolyTC;		// microseconds

	double
		m_dJumpSpeed,		// units per second
		m_dMarkSpeed;		// unsit per second

	// List Control Flags
	UINT
		m_nListSize,
		m_nListFlushLimit,
		m_nMaxListSize;
	BOOL
		m_bUsingList1;
	UINT
		m_nArcPolyLength;
	UINT
		m_nPrevPeriod,
		m_nDelay;

	UINT
		m_nListPosition;

	short
		m_siLastX,
		m_siLastY,
		m_siLastZ;

	unsigned short
		m_nRasterPeriod;
	UINT
		m_nRasterListSize;

	unsigned short
		m_usDigOut;	// Shadow for output port

	BOOL
		m_bInRasterMode;

	BOOL
		m_bAborted;

	double
		m_dWobbleAmp,
		m_dWobbleFreq;


// Operators
public:

	CRTCSA4Driver (void);
	~CRTCSA4Driver (void);

	void Init (void);

	UINT InitCard(void);

	void GetData(CRTCSA4Data *pData);
	BOOL SetData(CRTCSA4Data *pData);
	void GetState(CRTCSA4State *pState);
	void UpdateState(void);
	BOOL IsWorking(void);
	UINT GetCardCount(void);
	UINT GetCurrentCard(void);
	UINT SetCurrentCard(UINT nCard);
	UINT GetVersionHardware(void);
	UINT GetVersionSoftware(void);
	UINT GetVersionDLL(void);
	UINT GetVersionSerialNumber(void);
	void GetOptions(void);
	BOOL IsOptFlyCapable(void);
	BOOL IsOpt2HeadsCapable(void);
	BOOL IsOpt3DCapable(void);
	BOOL IsOptIOCapable(void);

	int LoadCorrectionFile(UINT nTable, BOOL bNoGain = TRUE);
	int LoadAllCorrectionFiles(void);
	void SetCorrectionTables(BOOL bSetup = FALSE);
	int LoadProgramFile(void);

	void SetControlMode(void);
	void SetListMode(void);
	UINT IsBusy(void);
	BOOL WaitOnList(UINT nList = 0);
	void Abort(void);
	UINT Pause(int iDelay = -1);
	void SetDelayMode(void);
	void SetScannerDelays(void);
	void SetScannerSpeed(void);
	void SetLaserMode(void);
	void SetLaserDelays(void);
	void SetLaserStandby(BOOL bOn);
	void SetLaserFPS(BOOL bOn);
	void SetLaserTiming(void);
	void SetAnalogPort(unsigned short nPort, unsigned short nValue);
	void SetDIOPort(BYTE byValue);

	void CenterGalvos(void);
	void LaserOn(double dOnTime = 0.0);
	void LaserOff(void);

	BOOL OpenList (void);
	void FlushList (void);

	BOOL MoveGalvos (short *, short *, short *, short *, long *, short *);
	void AddMove (short siX, short siY);
	void AddLine (short siX, short siY);
	void AddArc(long *lVecInfo, short *siVecInfo);
	BOOL GetNextArcSegment(long *lVecInfo, short *siVecInfo, short *siX, short *siY, UINT m_nArcPolyLength);
	void TransformXYZ(short *psiX, short *psiY, short *psiZ);
	void TransformXYZ(double *pdX, double *pdY, double *pdZ);
	BOOL MoveGalvosRaster(short *, short *, short *, short *, long *, short *, BYTE *);
	BOOL MoveGalvosRaster8(short *, short *, short *, short *, long *, short *, BYTE *, BYTE *);
	void AddRasMove (short siX, short siY);
	void AddRasDot (short siDX, short siDY, short siSteps, unsigned short *pwData);
	void Spot(short *, short *, short *);

	void GotoPos(short x, short y, short z = 0);
	void GetPos(short *x, short *y, short *z = NULL);
	void JumpAbs(short x, short y, short z = 0);
	void MarkAbs(short x, short y, short z = 0);
	void SetPixelLine(short dx, short dy, unsigned short N, unsigned short Period, unsigned short *Ptr);
	void SetPixel(unsigned short Data, unsigned short ADC);

	void GetLastPos(short *x, short *y, short *z = NULL);
	void SetLastPos(short x, short y, short z = 0);
	
	unsigned short SetRasterSpeed(UINT nPeriod);
	UINT SetRasterListSize(UINT nPixels);

	void SetConfigValues(CRTCSA4Data *pData);
	void SetConfigFieldSize(double dSize);
	void SetConfigDelayValues(double dJumpTC, double dMarkTC, double dPolyTC, double dLaserOffTC, double dLaserOnTC);
	void SetConfigLaserValues(double dTicklePeriod, double dTickleWidth, double dQSWidth, double dFPSWidth);
	void SetSpeedValues(double dJump, double dMark);
	void SetSpeed(int iPosRes, double dFieldSize, double dMarkSpeed);
	void SetPowerAndPeriod(double dFraction, double dPeriod);
	void SetLaserPower(double dFraction);
	void SetDelays(double dStart, double dPoly, double dAngle, double dEnd, double dPos, double dDist,double dDistance);
	
	BOOL GetFlyCounts(long *pCountsX, long *pCountsY);
	BOOL SetFlyKX(double *pX);
	BOOL SetFlyKY(double *pY);
	void SetEncoderValues(BOOL bEnable);

	double Get3DZOffset(void);
	void Set3DZOffset(double dZOffset);

	BOOL IsExtStartSignal(void);
	BOOL IsExtStopSignal(void);

	BOOL EnableExtStartSignal(BOOL bEnable);
	BOOL SetNullLists(void);
	BOOL ClearExtStopSignal(void);

	BOOL ReadDigIns(BYTE *pIn);
	BOOL WriteDigOuts(BYTE byOut);
	BOOL GetDigOuts(BYTE *pOut);

	BOOL SetLaserExtension(BYTE byValue);

	void SetLaserTickle(BOOL bOn);
	void SetRasterMode(BOOL bOn);

	BOOL IsExtStartStopEnabled(void);
	BOOL IsIOExtensionEnabled(void);

	UINT GetListPosition(void);
	BOOL IsInList1(void);
	BOOL IsInList2(void);
	UINT GetList(void);


	void SetWobble(BOOL bOn);
	void SetWobbleValues(double dWidth, double dFreq);

	BOOL IsCTBFile3D(CString sPath);

	BOOL IsFlyXEnabled(void);
	BOOL IsFlyYEnabled(void);

	int GetFlySimAxis(void);
	void SetFlySimAxis(int iAxis);

	void ResetFlyEncoder(BOOL bReset);

	UINT MarkSpecialRaster(CObject *pItem, long lPosX, long lPosY);
	void JumpRel(short x, short y, short z = 0);
	void MarkRel(short x, short y, short z = 0);

	BOOL SetFocus(unsigned short usFocus);

	int GetDIOMode(void);

	int ClearEncoderValues(void);

	void SetScannerCableLength(void);

	void SetGalvoPosition(short sX, short sY);

	void LoadVarPolyDelay(void);

	BOOL ReadDigInsEx(unsigned short *pIn, BOOL bOverride);
	BOOL WriteDigOutsEx(unsigned short byOut, BOOL bOverride);
	BOOL GetDigOutsEx(unsigned short *pOut, BOOL bOverride);

	BOOL m_bLogging;
	int m_iLogFileIndex;
	int m_iLogFileCallCount;
	CStdioFile m_LogFiles[RTCSA4_LOGFILE_MAXFILES];
	void LogFileStart(void);
	void LogFileCreateFolder(void);
	void LogFilePurge(void);
	void LogFileAdvance(void);
	void LogFileOpen(void);
	void LogFileAdd(CString sFunc, CString Arg, CString Result);
	BOOL LogFileTest(CString sFunc);
	void LogFileClose(void);

	BOOL IsConnected(void);

	//// 1.1063 PAT 07.10.05 RTC SCANALONE CHANGES ////
	void ClearLists(void);
	//// 1.1063 ///////////////////////////////////////

	//// 1.1335 PAT 15.06.07 UPDATE SPOT CODE ////
	void SpotX(double dDuration);
	//// 1.1335 //////////////////////////////////

	//// JT080128A Adding Custom Pulse Width Table for RTC4 ////
	bool	m_bPTLeadingEdge;
	BOOL	m_bPulseTableOn;
	double	m_adPulseTable[PT_MAX_ENTRIES];
	int		m_iPulseTableArraySize;
	double	m_dRestartDelay;

	void SetConfigLaserPulseTable( BOOL bOn, int iLength, double *pArray, double dRestart, bool bLeadingEdge );
	void SetCustomPulseTable();
	//// JT080128A /////////////////////////////////////////////

	//// PS080204B SPI LASER ////
	int m_iLaserType;
	int SetLaserType(int iType);

	int m_iWaveform;
	BOOL	SetLaserWaveform(int iWaveform);
	//// PS080204B //////////////

	//// PS080318A RTC Q WIDTH CONTROL ////
	double m_dQSWidthConfig;
	void SetLaserPeriodAndWidth(double dPeriod, double dWidth);
	//// PS080318A ////////////////////////

	//// PS080718A PULSE TABLE ANALOG MODE ////
	int m_iPTMode;
	double m_dPTFinal;
	void SetConfigLaserPulseTable2( BOOL bOn, int iMode, int iLength, double *pArray,
			double dRestart, bool bLeadingEdge , double dFinal);
	//// PS080718A ////////////////////////////

	//// PS090128A PREMOVE DELAY FOR DAC ////
	double	m_dPreJumpTC;	// microseconds
	void AddDelay(double dDelay);
	//// PS090128A //////////////////////////

	//// PS090417A RTC CALL LOGGING FILTERING ////
	int	m_iLogFilePrevCall;
	void LogFileEnd(void);
	//// PS090417A ///////////////////////////////

	//// PS090420A PREMOVE FOR RTC ////
	bool m_bLaserWasOn;
	//// PS090420A ////////////////////

	//// PS090924A ANALOG FREQ CONTROL IN RTC ////
	void SetLaserFreq(double dFraction);
	//// PS090924 ////////////////////////////////

	//// PS091203E ANALOG FREQ VS PULSE TABLE ////
	int GetPulseTablePort(void);
	//// PS091203E ///////////////////////////////

	//// PS20100427B SIMMER CURRENT SETTING ////
	void SetLaserPowerSimmerFraction(double dFraction);
	//// PS20100427B ///////////////////////////

protected:

#include ".\RTCSA4DLL_DEC.h"

protected:
};

#endif // RTCSA4DRIVER
