// Cyclopedia.h
//
// Header file for the CCyclopedia class
/////////////////////////////////////////////////
/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
/////////////////////////////////////////////////
#ifndef CCYCLOPEDIA
#define CCYCLOPEDIA

#include <afx.h>
#include "GSet.h"

/*** 1.0129 PAT 21.12.98 NEW BITMAPS ***/
#define CYC_HASH_NAMESEP	'~'
#define CYC_HASH_SEP		'^'
/*** 1.0129 ****************************/

class CCyclopedia : public CObArray
{
// Attributes
private:

	// Temporary pointers to a set used to avoid
	// searching when doing multiple access of the same data
	CGSet	*m_pTempSet;


	/// 1.0202 DAN 4/14/99 Null Character FixAngle ///
	// Empty Vector List
	// this needs to be long enough to skip the first 9 entries
	long m_NullVecList[VG_HS_FONT+1];
	////1.0202////////////////////////////////////////

// Operators
public:

	// Construction / Destruction
	CCyclopedia(void);
	void Init(void);		// Initialization

	~CCyclopedia(void);
	void RemoveAll(void);	// removes all elements

	// Hashing function -- Override in derived classes
	CString Hash(void);


	// Data Access
	CGSet* GetAt(int nIndex) const;

	// Returns the set matching sHash
	CGSet* FindSet(CString sHash);
	// Returns cSymbol in the set matching sHash
	CGSymbol* FindSymbol(CString sHash, _TUCHAR cSymbol = NULL);
	// Returns the header of cSymbol in the set matching sHash
	ClGHeader* GetHeader(CString sHash, _TUCHAR cSymbol = NULL);
	// Returns the vector list of cSymbol in the set matching sHash
	long* GetVecList(CString sHash, _TUCHAR cSymbol = NULL);

	// Puts the temp set pointer to the set matching sHash
	// This is for performance improvement.  Do not store for later use
	CGSet* SetCurrentSet(CString sHash);
	// Access the temp pointer to get cSymbol
	CGSymbol* FindSymbol(_TUCHAR cSymbol = NULL);
	// Access the temp pointer to get the header
	ClGHeader* GetHeader(_TUCHAR cSymbol = NULL);
	// Access the temp pointer to get the vector list
	long* GetVecList(_TUCHAR cSymbol = NULL);

	// Returns the position of the set
	int GetSetPos(CString sHash);
	// Removes the set matching sHash
	BOOL RemoveSet(CString sHash);
	// Removes the symbol from the set
	BOOL RemoveSymbol(CString sHash, _TUCHAR cSymbol = NULL);

	// Adds a symbol to the set
	BOOL AddSymbol(CGSymbol *pSymbol, CString sHash, _TUCHAR cSymbol = NULL);
	// Adds a symbol to the temp set pointer
	BOOL AddSymbol(CGSymbol *pSymbol, _TUCHAR cSymbol = NULL);

	// Creates a set and adds it to the cyclopedia
	BOOL AddSet(CString sHash);

	// Returns the hash key of the current set
	CString GetCurrentHash(void);

	/*** 1.0046 DAN 10.7.98 GRAPHIC MEM SAVE ***/
	void SetUsedFlag (CString sHash);
	/*******************************************/

	//// 1.0313 PAT 29.03.00 UPDATING GLIBINFO ////
	int GetCurSetSize(void);
	//// 1.0313 ///////////////////////////////////

};

#endif