//  Header file for CGLibrary class
//
// American LaserWare, Inc
// Pat Summers
// 13 SEP 1997
//
/****
	The GLibrary class centers around a list of vector
	graphic data (longs) stored in the CLongList member.
	The other member data and functions serve to provide
	access to the data in the list as well as to initialize
	and load the list with the required data from files. Hell
****/
//
///////////////////////////////////////////////////////////

// glibrary.h

#ifndef CGLIBRARY
#define CGLIBRARY

// Include header files required for members
#ifdef _ALIUSE
	#include "..\vgraph\vgraph.h"
	#include "..\WSFileInfo\thumb.h"
	#include "..\WSFileInfo\filecons.h"
/*** 1.0067 PAT 02.06.98 MORE RELATIVE / ABSOLUTE PATH ***/
	#include "..\WSData\item.h"
/*********************************************************/
//// 1.0307 PAT 23.03.00 TPO ADDITION TO VGRAPH ////
	#include "..\VGraph\TPO.h"
//// 1.0307 ////////////////////////////////////////
#else
	#include "vgraph.h"
	#include "thumb.h"
	#include "filecons.h"
	#include "item.h"
//// 1.0307 PAT 23.03.00 TPO ADDITION TO VGRAPH ////
	#include "TPO.h"
//// 1.0307 ////////////////////////////////////////
#endif



/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
#include "Cyclopedia.h" 
#include "GFill.h"
#include "GBarFill.h"
/***********************************************/


// Class specific constant definitions
// ERRORX is the default failure number -1
#define GL_ERRORX					-1
// ERRORZ ss the default failure null character (ascii 0)
#define GL_ERRORZ					0
// Set max number of graphic items (max size of GraphList)
#define GL_GRAPHIC_MAX				998
// Set max number of fonts (max size of FontList)
#define GL_FONT_MAX					20
// Set max number of characters per font
#define	GL_FONT_MAXCHARS			256
// Set max number of handles = max graphs + max fonts * char per font
#define	GL_HANDLE_MAX				6120
// Set max number of vector graphic data
#define GL_LIST_MAX					1000000
// Set GIList index for default null symbols
#define	GL_INDEX_NULLGRAPHIC		0
#define GL_INDEX_NULLCHAR			4
#define GL_INDEX_INITVALUE			14
// Set HandleList to point to null symbols
#define GL_HANDLE_NULLGRAPHIC		0
#define GL_HANDLE_NULLCHAR			1
#define GL_HANDLE_INITVALUE			2
/*** 1.0217 PAT 15.07.99 NULL CHARACTER FIX ***/
#define GL_HANDLE_NULLSYMBOL		0  
/*** 1.0217 ***********************************/
// Enumberate load error types
#define	GL_LE_NONE					0
#define GL_LE_BADFILE				1
#define GL_LE_BADHEADER				2
#define	GL_LE_BADVECTOR				3
#define GL_LE_BADCHAR				4
#define	GL_LE_MEMORY				5

/*** 1.0067 PAT 02.06.98 MORE RELATIVE / ABSOLUTE PATH ***/
//#define GL_DEFAULT_FONT				"SIMPLEX.ALF"
//#define GL_DEFAULT_BOX				"BOX.ALG"
//#define GL_DEFAULT_CIRCLE				"CIRCLE.ALG"
/*********************************************************/

//// 1.1255 PAT 06.11.04 IMAGESTREAM NAME REMOVAL ////
// Changed the text for reason 1 below to NOT reference
// ImageStream by name.
//// 1.1255 //////////////////////////////////////////
//// 1.0317 PAT 30.03.00 NO VECTORS ////
#define GLIB_ERRMSG_NOVECTORS	_T("No vector information was found in the specified file!\n\n\
Possible reasons for this include:\n\
   1. File version is beyond the importer's capabilities.\n\
      Try saving the file as an earlier version.\n\n\
   2. File contains no vector data.\n\
      Some formats contain both bitmap and vector data.\n\
      Ensure that the image is vector based or switch to\n\
      a raster graphic type.\n\n\
   3. File data is corrupt or unknown.\n\
      Try re-exporting file as HPGL or PLT from your\n\
      graphic drawing package.")
/// 1.0317 ////////////////////////////

//// 1.0321 PAT 04.04.00 TOG EXTENSION ////
#define GLIB_OPT_PREFIX			_T("OPT-")
//// 1.0321 ///////////////////////////////

// Graphic library class definition
class CGLibrary : public CObject
{  
// Attributes
public:
	int
		m_iSizeOfGraphList,
		m_iSizeOfFontList,
		m_iFontHandle,
	 	m_iPrevVector,
		m_bValid,
		m_nLoadErrorType;
	long
		m_lHandle,
		m_lAddress,
		m_lInsertPoint,
		m_lStartingPoint,
		m_alHandleList[GL_HANDLE_MAX];
	_TXCHAR
		m_sComment[WS_FC_SIZE];
	CString
		m_asGraphList[GL_GRAPHIC_MAX];
	CString
		m_asFontList[GL_FONT_MAX];
	ClGHeader
		m_Header;
	ClGVector
		m_Vector;
	CLongList
		* m_pList;
	UINT
		m_nFileID,
		m_nVersion;
	CTime
		m_tTimeStamp;
	CThumbnail
		m_Thumbnail;
	CString
		m_sRoot,	// Absolute path to app's exe file
		m_sTemp;	// Relative path from root to temp subdir

	/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
	CCyclopedia	m_Fonts;
	CGFill		m_FontFills;
	CCyclopedia	m_Graphics;
	CGBarFill	m_BarFills;

	// This will be changed soon... used to
	// determine the size of the symbols prior 
	// to loading the font
	long		m_lSymbolSizes[256];

	/***********************************************/

	/// 1.0244 DAN 9/7/99 GRAPHIC LIBRARY STATISTIC INFORMATION ///
	CDialog *m_pGLibDlg;
	////1.0244/////////////////////////////////////////////////////


	//// 1.0307 PAT 23.03.00 TPO ADDITION TO VGRAPH ////
	CTPO
		m_TPO;
	//// 1.0307 ////////////////////////////////////////


// Implementations
public:
	CGLibrary();					// Constructor
	~CGLibrary();					// Destructor
	void Init(void);				// Initialize members
	int GetGraphName(CString*, long);
									// Obtains the name of a graphic
	int GetFontName(CString*, long);// Obtains the name of a font
	int GetSizeOfGraphList(void);	// Returns the number of graphics
	int GetSizeOfFontList(void);	// Returns the number of fonts	
	long GetInsertPoint(void);		// Returns the current insert point
	int GetLoadErrorType(void);		// Returns the current LoadErrorType
	long GetGraphicHandle(CString);	// Returns the handle for a graphic
	int GetFontHandle(CString);		// Returns the handle for a font
	long GetCharHandle(_TUCHAR);		// Returns the handle for a character
	int GetHeader(long, ClGHeader*);// Obtains header data for a symbol
	int GetVector(ClGVector*);		// Obtains the symbol's next vector
	int GetPrevVector(ClGVector*);	// Obtains the previous vector

	/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
	/*** 1.0171 PAT 27.04.99 ICS/MQZ BARCODE FIX ***/
	// Add two parameters to function
	//void ModifyBCFont(CString, long, long *, long *, long, long);
	void ModifyBCFont(CString, long, long *, long *, long, long, long, long);
										// Changes header and vectors of the
										// common BC font to the proper 
										// size for the current item
	/*** 1.0171 ************************************/
	void Modify2DFont(CString, long);
	/***********************************************/

	/*** 1.0129 PAT 21.12.98 NEW BITMAPS ***/
	void ModifyBmpGraphic(CString, int, int);
									// Changes header and vectors of the
										// common BMP file to the size of
										// the image in pixels
	/*** 1.0129 ****************************/

	/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
	BOOL SelectFont (CString);
	//// 1.0307 PAT 23.03.00 TPO ADDITION TO VGRAPH ////
	// Added to parameter list
	// BOOL SelectGraphic(CString);
	BOOL SelectGraphic(CString sGraphicName, BOOL bTPO);
	//// 1.0307 ////////////////////////////////////////

	BOOL LoadFontSet (CString);
	CGSymbol *LoadSetSymbol(CArchive& ar, _TUCHAR Ordinal);
	BOOL GetSymbolHeader(_TUCHAR Ordinal, ClGHeader* lGHeader, CCyclopedia *pCyclopedia);
	BOOL GetCharHeader(_TUCHAR Ordinal, ClGHeader* lGHeader);
	BOOL GetGraphHeader(ClGHeader* lGHeader);

	void GetSymbolSizes(CString sFilePath);
	long GetFontSymbolSize(CArchive& ar, _TUCHAR Ordinal);

	BOOL LoadGraphic(CString sGraphicName);
	void GetGraphicSize(CString sFilePath);
	long GetGraphicSymbolSize(CArchive& ar);
	/***********************************************/

	/*** 1.0053 PAT 21.05.98 RELATIVE/ABSOLUTE PATHS ***/
	CString MakeAbsolute(CString);
	/***************************************************/

///XXX
	int
		m_iGraphicSets,
		m_iGraphicSymbols,
		m_iFontSets,
		m_iFontSymbols,
		m_iFillSets,
		m_iFillSymbols,
		m_iBarSets,
		m_iBarSymbols,
		m_iTotalSets,
		m_iTotalSymbols;
///XXX

	//// 1.0247 PAT 11.11.99 MORE RELOAD ////
	void GetFileNameParts(CString sFilePath,
						  CString& sDir, CString& sName,
						  CString& sTitle, CString& sExt);
	int GetFileType(CString sFilePath);
	int GetFileTempType(CString sFilePath);
	BOOL IsFontFile(CString sFilePath);
	//// 1.0321 PAT 04.04.00 TOG EXTENSION ////
	// Added a parameter
	//int	RemoveTempFile(CString sOrgFile);
	int RemoveTempFile(CString sOrgFile, BOOL bTPO = FALSE);
	//// 1.0321 ///////////////////////////////
	//// 1.0247 /////////////////////////////


protected:
	long ListSearch(CString, CString[], long);
									// Search given list for given string
	long GetGraphic(CString);
    int GetFont(CString);
	int LoadGraph(void);			// Loads a graphic from a file
	CString GetProperPath(CString);	// Determines what file to load from
	void DoFileConversion(CString, CString, CString);
									// Converts the file as necessary
	int LoadFont(void);				// Loads a font from a file
	void LoadSymbol(CArchive&);		// Loads any symbol from a file
	BOOL ListIsFull(void);			// Returns whether list is full
	BOOL ValidHeader(void);			// Returns whether header seems valid
	BOOL ValidVector(void);			// Returns whether vector seems valid
	BOOL ValidChar(int);			// Returns whether character is valid
	long GetSymbolAddress(long);	// Returns the symbol's address in the
										// list referenced by a handle
	BOOL ValidHandle(long);			// Returns whether a handle is used

	/*** 1.0223 PAT 03.08.99 NO TEMP GRAPHIC OPTION ***/
	void CheckQuickLoad(CString sFilePath, BOOL bTPG = TRUE);
	/*** 1.0223 ***************************************/
	
	/// 1.0266 DAN 12/16/99 CLEANUP OF GRAPHIC MEMORY ///
public:
	BOOL RemoveFill(CString sHashKey);
	////1.0266///////////////////////////////////////////

	//// 1.0307 PAT 23.03.00 TPO ADDITION TO VGRAPH ////
	BOOL DoTPO(CGSymbol *pSymbol, BOOL bFill = FALSE, long lFlags = 0x00000000);
	//// 1.0307 ////////////////////////////////////////

	//// 1.0313 PAT 29.03.00 UPDATING GLIBINFO ////
	
	//// 1.0321 PAT 04.04.00 TOG EXTENSION ////
	// Added a parameter
	//BOOL RemoveGraphic(CString sHashKey);
	BOOL RemoveGraphic(CString sHashKey, BOOL bTPO = FALSE);
	//// 1.0321 ///////////////////////////////
	
	BOOL RemoveFont(CString sHashKey);
	//// 1.0313 ///////////////////////////////////

	//// 1.0321 PAT 04.04.00 TOG EXTENSION ////
	BOOL
		m_bTPO,
		m_bUseOptTempFile;

	BOOL SaveGraphic(CGSymbol *pSymbol, CString sFileName, BOOL bIsTempForOpt = FALSE);
	//// 1.0321 ///////////////////////////////


	//// 1.0317 PAT 30.03.00 NO VECTORS ////
	BOOL IsReservedFile(CString sFilePath);
	CString MakeRelative(CString sPath);
	//// 1.0317 ////////////////////////////

	//// 1.0667 PAT 29.04.02 CONFIRM LINKS UPDATE ////
	BOOL IsGraphicResident(CString sFile);
	BOOL IsFontResident(CString sFile);
	//// 1.0667 //////////////////////////////////////

	//// 1.0677 PAT 24.05.02 DATA MATRIX BITMAP ////
	void Modify2DCodeGraphic(CString, int, int);
	//// 1.0677 ////////////////////////////////////

	long GetRealHeight(CString sFileName, BOOL bTPO);
	long GetRealWidth(CString sFileName, BOOL bTPO);
	long GetRealXPos(CString sFileName, BOOL bTPO);
	long GetRealYPos(CString sFileName, BOOL bTPO);

	BOOL IsRealGraphic(CString sFileName, BOOL bTPO);

	//// 1.0741 PAT 22.01.03 TTF AS STRINGS ////
	void ModifyATNFont(CString sFont, long lHeight, long *BarWidth,
							 long *SpaceWidth, long lLQZ, long lRQZ,
							 long lMQZ, long lICS);
	//// 1.0741 ////////////////////////////////

	//// 1.0699 PAT 12.08.02 DRILL OBJECT IN GUI ////
	BOOL LoadStockGraphicSpot(void);
	//// 1.0699 /////////////////////////////////////

	//// 1.0981 PAT 26.01.05 GRAPHIC ASPECT RATIO UPDATE ////
	long GetGraphicHeaderHeight(CString sFileName, BOOL bTPO);
	long GetGraphicHeaderLength(CString sFileName, BOOL bTPO);
	//// 1.0981 /////////////////////////////////////////////

	//// PS090304A TRANSLATOR CLASS EXCISING ////
	CString GetErrMsgNoVectors(void);
	//// PS090304A //////////////////////////////

// Operators

};

#endif
                          
